/********************************************************************************************************
	SOMIM, a program that can calculate the maximum mutual information and the corresponding optimal POVMs given a set of statistical operators.
	Copyright (C) 2007, 2010 K.L. Lee, J.W. Shang, W.K. Chua, S.Y. Looi and B.-G. Englert.

	This program is a free software: you can redistribute it and/or modify it under the terms of the GNU General Public License Version 3 as published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. 
    See the GNU General Public License for more details.

	You can refer to http://www.gnu.org/licenses/ for the GNU General Public License.
	
	Detail manual for the program could be downloaded from http://www.quantumlah.org/publications/software/SOMIM/.
    
    The code for Graphical User Interface (GUI) is following the book 'Cross-Platform GUI Programming with WxWidgets'.
********************************************************************************************************/
//******************************************************************************************************/
//	Compiled under Windows XP Service Pack 3, with wxWidgets 2.8.10, MinGW 5.1.6, MSYS 1.0.11
//
//	Compile with the following command
//		g++ MI.cpp `wx-config --libs` `wx-config --cxxflags` -o SOMIM
//	in MSYS shell.
//	
//  If you face problems running the program in a Linux environment, try
//      export LD LIBRARY PATH=/usr/local/lib
//  in command window.
//
//******************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string> /* for string variable */
#include <vector> /* for manipulating variable matrix during runtime */
#include <complex> /* for complex double, creal, cimag, csqrt, complex arithmetic */
#include <cmath> /* for log */
#include <ctime> /* for time */

//--------------------------------------------------------------
//	import headers for GUI
//--------------------------------------------------------------
#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/stattext.h>
#include <wx/statbox.h>
#include <wx/spinctrl.h>
#include <wx/grid.h>
#include <wx/radiobox.h>
#include <wx/filedlg.h>
#include <wx/textfile.h>
#include <wx/tokenzr.h>
//--------------------------------------------------------------
//	end of import of headers for GUI
//--------------------------------------------------------------

using namespace std;
// define complex double data type
typedef complex<double> dcomp;

#include "functionMI.h" /* class header files for functions calculating mutual information */
#include "MI.h" /* class header files for GUI */

IMPLEMENT_APP(myApp);

//----------------------------------------------------------
//	functions that do not belong to any class
//----------------------------------------------------------
// convert a string into a double
double convertString(wxString myString){
	double myDouble;
	if(myString.ToDouble(&myDouble)){
		return myDouble;
	};	
}	

//----------------------------------------------------------
//	Event handlers for myAPP
//----------------------------------------------------------
bool myApp::OnInit()
{
	myFrame *myframe = new myFrame(_("SOMIM Version 2.0"), wxDefaultPosition, wxSize(300,200));
	myframe->Show(true);
	SetTopWindow(myframe);
	
	return TRUE;
}

//----------------------------------------------------------
//	matrixGrid
//----------------------------------------------------------
matrixGrid::matrixGrid(wxPanel *panel, int wxID)
	   : wxGrid(panel,wxID,wxDefaultPosition,wxSize(350,80))
{	
	CreateGrid(3,3);
	SetRowLabelSize(0);
	SetColLabelSize(0);
	SetDefaultColSize(60,true);
	SetDefaultRowSize(20,true);
	
	// set to display only floating point numbers
	for(int i=0;i<3;i++)
		SetColFormatFloat(i,-1,4);
}

// modify the number of columns and rows displayed
void matrixGrid::setDimension(int x)
{
	int j;
	int i=GetNumberCols();
	if(x>i){
		// need to increase the number of columns and the number of rows
		InsertRows(i, x-i, true);
		InsertCols(i, x-i, true);
	}else if(x<i){
		// need to decrease the number of columns and the number of rows
		DeleteRows(0, i-x, true);
		DeleteCols(0, i-x, true);
	}else{
		// do nothing since the rho has the same dimension
	}	
	// set to display only floating point numbers	
	for(i=0;i<x;i++)
		SetColFormatFloat(i,-1,4);
}

// modify the value at the (i,j)th cell to the value f
void matrixGrid::changeElement(int i, int j, double f)
{
	wxString s = "";
	s += wxString::Format(_T("%f"),f);
	
	SetCellValue(i,j,s);
}

// display the matrix a of dimension m X n
void matrixGrid::setGrid(vector< vector<dcomp> > a, int m, int n, int ri)
{
	int i,j;
	wxString s;
	
	for(i=0;i<m;i++){
		for(j=0;j<n;j++){			
			s = "";
			if(ri==0) s += wxString::Format(_T("%f"),real(a[i][j])); // real part
			if(ri==1) s += wxString::Format(_T("%f"),imag(a[i][j])); // imginary part
			SetCellValue(i,j,s);
		}
	}
}
//----------------------------------------------------------
//	myPanel
//----------------------------------------------------------

//----------------------------------------------------------
//	Implementation of GUI
//----------------------------------------------------------
myPanel::myPanel( wxFrame *frame, int x, int y)
       : wxPanel( frame, wxID_ANY, wxPoint(x, y))
{
    // initialisation of variables
	my_panel = NULL;
	m_book = NULL;
	m_calcButton = NULL;
	m_JTEXT = NULL;
	m_KTEXT = NULL;
	m_rhoDimension = NULL;
	m_rhoDimensionText = NULL;
	m_directGradChance = NULL;
	m_Inputrho = NULL;
	rhoGrid = NULL;
	povmGrid = NULL;
	m_rhoRadio = NULL;
	m_povmRadio = NULL;
	m_ranRadio = NULL;
	m_InputOrDisplayPOVM = NULL; 
	m_displayMI = NULL;
	m_tol_MI = NULL;
	m_output = NULL;
	m_directory = NULL;
	m_directChange = NULL;
	
    // to display "Real" or "Imaginary" radio button
	wxArrayString strings;
	strings.Add(wxT("&Real"));
	strings.Add(wxT("&Imaginary"));
	
    // to choose "Yes" or "No" radio button
	wxArrayString strings2;
	strings2.Add(wxT("&Yes"));
	strings2.Add(wxT("&No"));
	
	// Create the notebook with tabs
	m_book = new wxNotebook(this, BOOKCTRL);
	
 	my_panel = new wxPanel(m_book);
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL); 	
	 	 	
 	wxFlexGridSizer *topsizer = new wxFlexGridSizer(2,0,0);
 	topsizer->AddGrowableCol(1);
 	
 	//-------------------------------------------------------------------
 	//	sizer for first static box
 	//-------------------------------------------------------------------
 	// add the first paramenter input: number of rho_j
 	wxStaticText *text = new wxStaticText(my_panel, wxID_ANY, _T("Number of possible Quantum States J ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_JTEXT = new wxSpinCtrl(my_panel, wxID_JTEXT, _T("2"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 100, 2);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	topsizer->Add(m_JTEXT, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);

 	// add the second paramenter input: number of pi_k
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Number of POVM outcomes K ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_KTEXT = new wxSpinCtrl(my_panel, wxID_KTEXT, _T("3"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 100, 3); 	
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	topsizer->Add(m_KTEXT, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);

	// add the third parameter input: dimension of rho_j
	text = new wxStaticText(my_panel, wxID_ANY, _T("Dimension of statistical operators N ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	wxBoxSizer *temp_sizer = new wxBoxSizer(wxHORIZONTAL);
 	m_rhoDimension = new wxSpinCtrl(my_panel,wxID_rhoDimension, _T("3"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 80, 3);
 	m_rhoDimensionText = new wxStaticText(my_panel,wxID_rhoDimensionText, "3", wxDefaultPosition, wxDefaultSize);
 	temp_sizer->Add(m_rhoDimension, 0, wxGROW|wxALL, 0);
 	temp_sizer->Add(new wxStaticText(my_panel,wxID_ANY, _T("X"), wxDefaultPosition, wxDefaultSize), 0, wxALL, 5);
 	temp_sizer->Add(m_rhoDimensionText, 0, wxALL, 5);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	topsizer->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
	
	// add the fourth parameter input: percentage chance to use direct gradient in the steepest ascent method
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Percentage chance to calculate with direct gradient ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_directGradChance = new wxSpinCtrl(my_panel, wxID_ANY, _T("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100, 0);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	topsizer->Add(m_directGradChance, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);						  
 						  
 	// add the fifth parameter input: tolerance in MI
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Tolerance in Mutual Information ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_tol_MI = new wxTextCtrl(my_panel, wxID_ANY, _T("0.000001"), wxDefaultPosition, wxDefaultSize);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 	topsizer->Add(m_tol_MI, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
	  	
 	// add the sixth parameter input: the output file
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Output file ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_output = new wxTextCtrl(my_panel, wxID_ANY, _T("output.txt"), wxDefaultPosition, wxDefaultSize);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 	topsizer->Add(m_output, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
 	
 	// add the seventh parameter input: output file directory
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Save output file to"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_directory = new wxTextCtrl(my_panel, wxID_ANY, _T("C:\\"), wxDefaultPosition, wxDefaultSize);
 	m_directChange = new wxButton(my_panel, BUTTON_Save, _T("&..."), wxDefaultPosition, wxDefaultSize);
 	wxBoxSizer *temp_sizer3 = new wxBoxSizer(wxHORIZONTAL);
 	temp_sizer3->Add(m_directory, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
 	temp_sizer3->Add(m_directChange, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

	topsizer->Add(temp_sizer3, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 	 	
 	wxBoxSizer *staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Parameter Settings")), wxVERTICAL );
 	staticbox->Add(topsizer,wxSizerFlags().Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags().Expand().Border(wxALL, 0));
 	
 	//-------------------------------------------------------------------
 	//	sizer for second static box
 	//------------------------------------------------------------------- 	
	// add parameter to input the quantum states
	wxBoxSizer* topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_Inputrho = new wxSpinCtrl(my_panel, wxID_Inputrho, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 2, 1);
	m_rhoRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Display Real or Imaginary"), wxDefaultPosition, wxDefaultSize, strings, 2, wxRA_SPECIFY_COLS);	
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Input the matrix representation of ")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_Inputrho, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	temp_sizer->Add(m_rhoRadio, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);	
	rhoGrid = new matrixGrid(my_panel,wxID_rhoGrid);
		 	 																		    
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
	topsizer2->Add(rhoGrid, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Input the Matrix Representation of Quantum States")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	

 	//-------------------------------------------------------------------
 	//	sizer for third static box
 	//------------------------------------------------------------------- 	
	// add parameter to input the initial or to display the optimal POVMs
	topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_InputOrDisplayPOVM = new wxSpinCtrl(my_panel, wxID_InputOrDisplayPOVM, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 2, 1);
	m_povmRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Display Real or Imaginary"), wxDefaultPosition, wxDefaultSize, strings, 2, wxRA_SPECIFY_COLS);
    temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Input/Display ")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);	
    temp_sizer->Add(m_InputOrDisplayPOVM, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	temp_sizer->Add(m_povmRadio, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
	povmGrid = new matrixGrid(my_panel,wxID_povmGrid);
	povmGrid->EnableEditing(true); // enable or disable editing of the povm grid
	
	m_ranRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Randomize?"), wxDefaultPosition, wxDefaultSize, strings2, 2, wxRA_SPECIFY_COLS);
	temp_sizer->Add(m_ranRadio, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
	
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
	topsizer2->Add(povmGrid, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Input initial/Display optimized POVM outcomes")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	
 	
 	//-------------------------------------------------------------------
 	//	sizer for fourth static box
 	//------------------------------------------------------------------- 	
	// add parameter to display maximal mutual information and control button
	topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Maximal Mutual Information =")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	m_displayMI = new wxTextCtrl(my_panel, wxID_DisplayMI, wxEmptyString, wxDefaultPosition, wxDefaultSize);
	temp_sizer->Add(m_displayMI, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	wxBoxSizer *temp_sizer2 = new wxBoxSizer(wxHORIZONTAL);
	m_calcButton = new wxButton(my_panel, BUTTON_Calc,_T("&Calculate MI"), wxDefaultPosition, wxSize(100, 30));
	temp_sizer2->Add(m_calcButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer2->Add(new wxButton(my_panel, BUTTON_Reset,_T("&Reset"), wxDefaultPosition, wxSize(100,30)), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
	topsizer2->Add(temp_sizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Display the maximal Mutual Information")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	 	
 	
 	my_panel->SetSizer(sizer);
 	m_book->AddPage(my_panel, _T("Untitled"), true);

 	// place everything in a sizer and attach to the frame
 	sizer = new wxBoxSizer(wxVERTICAL);
 	sizer->Add(m_book,wxSizerFlags(1).Expand().Border(0));
 	SetSizer(sizer);
 	sizer->SetSizeHints(frame);
 	
 	m_information = new MutInf(m_rhoDimension->GetValue(), m_JTEXT->GetValue(), m_KTEXT->GetValue());
 	rhoGrid->setGrid(m_information->getRho(0), m_rhoDimension->GetValue(), m_rhoDimension->GetValue(), 0);
 	povmGrid->setGrid(m_information->getPOVM(0), m_rhoDimension->GetValue(), m_rhoDimension->GetValue(), 0); // new
}

BEGIN_EVENT_TABLE(myPanel, wxPanel)
	EVT_SPINCTRL(wxID_JTEXT, myPanel::ChangeRhoNumber)
	EVT_SPINCTRL(wxID_rhoDimension, myPanel::ChangeRhoDimension)
	EVT_SPINCTRL(wxID_KTEXT, myPanel::ChangePOVMNumber) // new
	EVT_SPINCTRL(wxID_rhoDimension, myPanel::ChangePOVMDimension) // new
	EVT_SPINCTRL(wxID_Inputrho, myPanel::UpdateRho)
	EVT_SPINCTRL(wxID_InputOrDisplayPOVM, myPanel::UpdatePOVM)
	EVT_MENU(wx_Calc, myPanel::OnCalc)
    EVT_GRID_CMD_CELL_CHANGE(wxID_rhoGrid, myPanel::OnRhoCellValueChanged)
    EVT_GRID_CMD_CELL_CHANGE(wxID_povmGrid, myPanel::OnPOVMCellValueChanged) // new
	EVT_BUTTON(BUTTON_Calc, myPanel::OnCalc)
	EVT_BUTTON(BUTTON_Reset, myPanel::OnReset)
	EVT_BUTTON(BUTTON_Save, myPanel::OnSave)	
END_EVENT_TABLE()

//----------------------------------------------------------------------
//	myFrame
//----------------------------------------------------------------------
#include "sample.xpm"
myFrame::myFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
		:wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
	SetIcon(wxIcon(sample_xpm));
		
	wxMenu *myFilemenu = new wxMenu;
	myFilemenu->Append(wxID_IMPORT, "&Import\tCtrl+I", "Import States");
	myFilemenu->Append(wxID_EXIT, "&Exit\tAlt+F4", "Quit");
	
	wxMenu *myCalcmenu = new wxMenu;
	myCalcmenu->Append(wx_Calc, "&Calculate\tCtrl+C", "Calculate Mutual Information");
	
	wxMenu *myAboutmenu = new wxMenu;
	myAboutmenu->Append(wxID_ABOUT, "&About", "About SOMIM");
	
	wxMenuBar *mymenubar = new wxMenuBar();
	mymenubar->Append(myFilemenu, "&File");
	mymenubar->Append(myCalcmenu, "&Calc");
	mymenubar->Append(myAboutmenu, "&About");
	
	SetMenuBar(mymenubar);
	
	// Create status bar with 3 fields for the window
	CreateStatusBar(2);
	SetStatusText(_T("Ready"), 0);

	// myPanel *panel = new myPanel(this, 0, 0);
	m_panel = new myPanel(this, 0, 0);
}

BEGIN_EVENT_TABLE(myFrame, wxFrame)
	EVT_MENU(wxID_IMPORT, myFrame::OnImport)
	EVT_MENU(wxID_EXIT, myFrame::OnQuit)
	EVT_MENU(wxID_ABOUT, myFrame::OnAbout)
END_EVENT_TABLE()

//-----------------------------------------------------------
//	Event Handlers for myFrame
//-----------------------------------------------------------
// to import the value of N, J, K from a file
int myFrame::extractParameter(wxString& text)
{
	int equalIndex = text.find("=");
	size_t stringLength = text.Len();
	wxString value = text.Mid(equalIndex+1, stringLength-equalIndex-1);
	value.Trim(true);
	value.Trim(false);
	
	int i;
	long l;
	if(value.ToLong(&l,10)){
		i=l;
		return i;
	}
}

// to import the matrix elements of each rho_j
void myFrame::extractRhoElement(wxString& text, int j, int& dimen)
{
	int i, k;
	int imindex, imsignindex, plusindex, minusindex;
	double reNo, imNo;
	wxString tempString, s;
	wxStringTokenizer dummyString;
	int equalIndex = text.find("=");
	size_t stringLength = text.Len();
	wxString value = text.Mid(equalIndex+1, stringLength-equalIndex-1);
	value.Trim(true);
	value.Trim(false);
	
	vector<wxString> row;
	wxStringTokenizer tkz(value, wxT("}")); // split the imported line into rows of matrix with "}" as the delimiter
	while(tkz.HasMoreTokens())
	{
    	wxString token = tkz.GetNextToken();
		token = token.AfterLast('{');
    	row.push_back(token);
	}

	m_panel->m_Inputrho->SetValue(j);
	for(i=0;i<dimen;i++){
		k=0;
		dummyString.SetString(row[i],",");
		while(dummyString.HasMoreTokens())
		{
			tempString = dummyString.GetNextToken();
			tempString.Trim(true);
			tempString.Trim(false);
			imindex = tempString.find("I");
			if(imindex != wxNOT_FOUND){ // there is an imaginary part of the number				
				// look for the starting position of the imaginary part
				plusindex = tempString.find("+",true); // look for + sign from the back
				minusindex = tempString.find("-", true); // look for - sign from the back
				if(plusindex == wxNOT_FOUND) plusindex = 0;
				if(minusindex == wxNOT_FOUND) minusindex = 0;
				imsignindex = (plusindex>minusindex || plusindex==minusindex)? plusindex:minusindex;

				value = tempString.Mid(imsignindex, imindex-imsignindex);
				value.Trim(true);
				value.Trim(false);
				imNo = convertString(value);
				if(imsignindex == 0){
					reNo = 0; // no real part
				}else{
					value = tempString.Mid(0,imsignindex);
					value.Trim(true);
					value.Trim(false);
					reNo = convertString(value);
				}
			}else{ // there is only real part
				reNo = convertString(tempString);
				imNo = 0;
			}
			
			if(m_panel->m_rhoRadio->GetSelection() == 0){ // display real part
				s = "";
				s += wxString::Format(_T("%f"),reNo);
				m_panel->rhoGrid->SetCellValue(i,k,s);
			}else{ // display imaginary part
				s = "";
				s += wxString::Format(_T("%f"),imNo);
				m_panel->rhoGrid->SetCellValue(i,k,s);
			}
			
			m_panel->m_information->setRhoElement(j-1,i,k,reNo,0);
			m_panel->m_information->setRhoElement(j-1,i,k,imNo,1);
			k++;
		}
	}
}

// to import the matrix elements of each pi_k
void myFrame::extractPOVMElement(wxString& text, int k, int& dimen)
{
	int i, j;
	int imindex, imsignindex, plusindex, minusindex;
	double reNo, imNo;
	wxString tempString, s;
	wxStringTokenizer dummyString;
	int equalIndex = text.find("=");
	size_t stringLength = text.Len();
	wxString value = text.Mid(equalIndex+1, stringLength-equalIndex-1);
	value.Trim(true);
	value.Trim(false);
	
	vector<wxString> row;
	wxStringTokenizer tkz(value, wxT("}")); // split the imported line into rows of matrix with "}" as the delimiter
	while (tkz.HasMoreTokens())
	{
    	wxString token = tkz.GetNextToken();
		token = token.AfterLast('{');
    	row.push_back(token);
	}

	m_panel->m_InputOrDisplayPOVM->SetValue(k);
	for(i=0;i<dimen;i++){
		j=0;
		dummyString.SetString(row[i],",");
		while(dummyString.HasMoreTokens())
		{
			tempString = dummyString.GetNextToken();
			tempString.Trim(true);
			tempString.Trim(false);
			imindex = tempString.find("I");
			if(imindex != wxNOT_FOUND){ // there is an imaginary part of the number				
				// look for the starting position of the imaginary part
				plusindex = tempString.find("+",true); // look for + sign from the back
				minusindex = tempString.find("-", true); // look for - sign from the back
				if(plusindex == wxNOT_FOUND) plusindex = 0;
				if(minusindex == wxNOT_FOUND) minusindex = 0;
				imsignindex = (plusindex>minusindex || plusindex==minusindex)? plusindex:minusindex;

				value = tempString.Mid(imsignindex, imindex-imsignindex);
				value.Trim(true);
				value.Trim(false);
				imNo = convertString(value);
				if(imsignindex == 0){
					reNo =0; // no real part
				}else{
					value = tempString.Mid(0,imsignindex);
					value.Trim(true);
					value.Trim(false);
					reNo = convertString(value);
				}
			}else{ // there is only real part
				reNo = convertString(tempString);
				imNo = 0;
			}
			
			if(m_panel->m_povmRadio->GetSelection() == 0){ // display real part
				s = "";
				s += wxString::Format(_T("%f"),reNo);
				m_panel->povmGrid->SetCellValue(i,j,s);
			}else{ // display imaginary part
				s = "";
				s += wxString::Format(_T("%f"),imNo);
				m_panel->povmGrid->SetCellValue(i,j,s);
			}
			
			m_panel->m_information->setPOVMElement(k-1,i,j,reNo,0);
			m_panel->m_information->setPOVMElement(k-1,i,j,imNo,1);
			j++;
		}
	}
}

void myFrame::OnImport(wxCommandEvent& e)
{
	// open a file dialog to import file
	wxString caption = _T("Choose a text file to import states");
	wxString wildcard = _T("TXT files (*.txt)|*.txt|Data files (*.dat)|*.dat|All files (*.*)|*.*");
	wxString defaultDir = _T("C:\\");
	wxString defaultFilename = wxEmptyString;
	wxString path = wxEmptyString;
	int filterIndex;
	wxTextFile file;
	
	wxFileDialog dialog(this,caption,defaultDir,defaultFilename,wildcard,wxOPEN);
	if(dialog.ShowModal() == wxID_OK){
		path = dialog.GetPath();
		filterIndex = dialog.GetFilterIndex();
		
		// read the file content and put into display
		if(file.Open(path))
		{
			int i,j;
			int dimen = extractParameter(file[0]);
			int number_J = extractParameter(file[1]);
			int number_K = extractParameter(file[2]);
			m_panel->m_rhoDimension->SetValue(dimen);
			m_panel->ChangeRhoDimensionFunction();
			m_panel->ChangePOVMDimensionFunction();
			m_panel->m_JTEXT->SetValue(number_J);
			m_panel->ChangeRhoNumberFunction();			
			m_panel->m_KTEXT->SetValue(number_K);
			m_panel->ChangePOVMNumberFunction();	
			for(i=0;i<number_J;i++){
				extractRhoElement(file[i+3],i+1,dimen);
			}
			
			if(m_panel->m_ranRadio->GetSelection()==0){// Yes, do nothing

			}else{// No, to import
			    for(j=number_J;j<(number_J+number_K);j++){
                    extractPOVMElement(file[j+3],j+1-number_J,dimen);
                }
			}
			m_panel->m_book->SetPageText(0,path.AfterLast('\\'));
		}
	}
}

void myFrame::OnQuit(wxCommandEvent& e)
{
	Close(TRUE);
}

void myFrame::OnAbout(wxCommandEvent& e)
{
    (void)wxMessageBox(_T("Search for Optimal Measurements using Iterative Methods (SOMIM), a program that can calculate the maximum mutual information and the corresponding optimal POVM outcomes given a set of statistical operators.\nCopyright (C) 2007, 2010 K.L. Lee, J.W. Shang, W.K. Chua, S.Y. Looi and B.-G. Englert."),
            _T("About SOMIM Version 2.0"), wxOK|wxICON_INFORMATION);
}

//-----------------------------------------------------------
//	Event Handlers for myPanel
//-----------------------------------------------------------

//-----------------------------------------------------------
//	Event Handlers to manipulate rhoGrid and povmGrid
//-----------------------------------------------------------
// change the number of rho_j
void myPanel::ChangeRhoNumber(wxSpinEvent& se)
{
	ChangeRhoNumberFunction();
}

// change the number of pi_k
void myPanel::ChangePOVMNumber(wxSpinEvent& se)
{
	ChangePOVMNumberFunction();
}

void myPanel::ChangeRhoNumberFunction()
{
	m_Inputrho->SetRange(1,m_JTEXT->GetValue());
	m_information->UpdateRhoDimension(m_JTEXT->GetValue(),m_rhoDimension->GetValue());
}

void myPanel::ChangePOVMNumberFunction()
{
	m_InputOrDisplayPOVM->SetRange(1,m_KTEXT->GetValue());
	m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());
}

// modify the dimension of each rho_j, update the rho_j matrix displayed
void myPanel::ChangeRhoDimension(wxSpinEvent& se)
{	
	ChangeRhoDimensionFunction();
}

// modify the dimension of each pi_k, update the pi_k matrix displayed
void myPanel::ChangePOVMDimension(wxSpinEvent& se)
{	
	ChangePOVMDimensionFunction();
}

void myPanel::ChangeRhoDimensionFunction()
{
	m_rhoDimensionText->SetLabel(wxString::Format(_T("%d"),m_rhoDimension->GetValue()));
	m_information->UpdateRhoDimension(m_JTEXT->GetValue(),m_rhoDimension->GetValue());
	rhoGrid->setDimension(m_rhoDimension->GetValue());
	
	int i;
	i = m_Inputrho->GetValue();	
	rhoGrid->setGrid(m_information->getRho(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_rhoRadio->GetSelection());	
}

void myPanel::ChangePOVMDimensionFunction()
{
	m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());
	povmGrid->setDimension(m_rhoDimension->GetValue());
	
	int i;
	i = m_InputOrDisplayPOVM->GetValue();	
	povmGrid->setGrid(m_information->getPOVM(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_povmRadio->GetSelection());	
}

// display the rho_j matrix selected by the user
void myPanel::UpdateRho(wxSpinEvent& e)
{
	int i;
	i = m_Inputrho->GetValue();
	rhoGrid->setGrid(m_information->getRho(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_rhoRadio->GetSelection());
}

// response to values entered by updating the corresponding rho_j
void myPanel::OnRhoCellValueChanged(wxGridEvent& ev)
{
	int row = ev.GetRow(),
        col = ev.GetCol();
        
    double d = convertString(rhoGrid->GetCellValue(row,col));
    m_information->setRhoElement(m_Inputrho->GetValue()-1,row,col,d, m_rhoRadio->GetSelection());  
}

// display the pi_k matrix selected by the user
void myPanel::UpdatePOVM(wxSpinEvent& e)
{
	int i;
	i = m_InputOrDisplayPOVM->GetValue();
	povmGrid->setGrid(m_information->getPOVM(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_povmRadio->GetSelection());
}

// response to values entered by updating the corresponding pi_k
void myPanel::OnPOVMCellValueChanged(wxGridEvent& ev)
{
	int row = ev.GetRow(),
        col = ev.GetCol();
        
    double d = convertString(povmGrid->GetCellValue(row,col));
    m_information->setPOVMElement(m_InputOrDisplayPOVM->GetValue()-1,row,col,d, m_povmRadio->GetSelection());  
}
//-----------------------------------------------------------
//	End of event handlers to manipulate rhoGrid and povmGrid
//-----------------------------------------------------------

void myPanel::OnCalc(wxCommandEvent& e)
{	
	this->Enable(false);
	m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());
	m_tol_MI->GetValue().ToDouble(&(m_information->tol_MI));
	m_information->output = m_directory->GetValue() + "\\" + m_output->GetValue(); // output file directory + name
	m_information->directGradChance = m_directGradChance->GetValue();
	m_information->calcMutualInformation();
	m_InputOrDisplayPOVM->SetRange(1,m_information->getK()); // to display the optimal POVMs
	povmGrid->setDimension(m_rhoDimension->GetValue());
	int i = m_InputOrDisplayPOVM->GetValue();
	povmGrid->setGrid(m_information->getPOVM(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_povmRadio->GetSelection());
	
	wxString s = "";
	s += wxString::Format(_T("%f"),m_information->MI);
	m_displayMI->SetValue(s);
	this->Enable(true);
}

//-----------------------------------------------------------
//	change the output file directory
//-----------------------------------------------------------
void myPanel::OnSave(wxCommandEvent& e)
{
	// open a file dialog to export file
	wxString caption = _T("Choose a directory to save the output file");
	wxString defaultDir = _T("C:\\");
	wxString path;
	
	wxDirDialog dialog(this, caption, defaultDir, wxDD_NEW_DIR_BUTTON);
	if(dialog.ShowModal() == wxID_OK){
        path = dialog.GetPath();
	}
    m_directory->SetValue(path); // set the new output file path
}

//-----------------------------------------------------------
//	reset all values to default
//-----------------------------------------------------------
void myPanel::OnReset(wxCommandEvent& e)
{
	m_rhoDimension->SetValue(3);
	m_JTEXT->SetValue(2);
	m_KTEXT->SetValue(3);
	m_directGradChance->SetValue(0);
	m_tol_MI->SetValue("0.000001");
	m_output->SetValue("output.txt");
	m_directory->SetValue("C:\\");
}
