/********************************************************************************************************
	SOMIM, a program that can calculate the maximum mutual information and the corresponding optimal POVMs given a set of statistical operators.
	Copyright (C) 2007, 2010 K.L. Lee, J.W. Shang, W.K. Chua, S.Y. Looi and B.-G. Englert.

	This program is a free software: you can redistribute it and/or modify it under the terms of the GNU General Public License Version 3 as published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. 
    See the GNU General Public License for more details.

	You can refer to http://www.gnu.org/licenses/ for the GNU General Public License.
	
	Detail manual for the program could be downloaded from http://www.quantumlah.org/publications/software/SOMIM/.
	
	The conjugate-gradient method, golden section search method as well as LU decomposition are performed following the algorithm given in 'Numerical Recipe in C', 2nd Ed. 
    This edition is available online at http://www.nrbook.com/c/.
    
    This reference will be referred to as "NRC" in the comments of the following codes.
********************************************************************************************************/
//******************************************************************************************************/
//	Compiled under Windows XP Service Pack 3, with wxWidgets 2.8.10, MinGW 5.1.6, MSYS 1.0.11
//
//	Compile with the following command
//		g++ MI.cpp `wx-config --libs` `wx-config --cxxflags` -o SOMIM
//	in MSYS shell.
//	
//  If you face problems running the program in a Linux environment, try
//      export LD LIBRARY PATH=/usr/local/lib
//  in command window.
//
//******************************************************************************************************/

class MutInf{
	//------------------------------------------------------------------------------
	//	variable declarations
	//------------------------------------------------------------------------------
	public:
		MutInf(int n, int j, int k);		
		void calcMutualInformation(); // main function to calculate the mutual information MI
				
		// variables
		int povmDimension, directGradChance, maxDimen, maxMatrix;
		double MI, new_MI, alpha, phi;
		double tol_MI, tol_alpha, tol_S;
		string output;
		vector< vector<double> > prob_table;
						
		// various functions that deal with actual calculation
		double calc_beta(vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		void calc_delta_A_dagger_CG(double, int, vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);		
		void calc_grad_MI(vector< vector<dcomp> >&, vector< vector< vector<dcomp> > >&, vector< vector<dcomp> >&);
		void calc_L_inverse(vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		inline double calc_MI(double some_alpha, vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);		
		double calc_MI_from_prob_table(vector< vector<double> >&);	
		void calc_new_A_dagger(double some_alpha, vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);		
		void calc_prob_table(vector< vector<dcomp> >&);
		void calc_R(vector< vector< vector<dcomp> > >&);
		void find_alpha_GSS(vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		void gen_A_dagger(vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		
		// functions for interaction with GUI
		int getK();
		vector <vector <dcomp> > getPOVM(int X);
		vector <vector <dcomp> > getRho(int X);
		void setRhoElement(int X, int Y, int Z, double d, int ri);
		void setPOVMElement(int X, int Y, int Z, double d, int ri);
		void UpdatePOVMDimension(int k, int n);
		void UpdateRhoDimension(int j, int n);				
		
		// functions initializing 2D and 3D matrices
		vector< vector<dcomp> > Init2DMatrix(int X, int Y);		
		vector< vector<double> > Init2DRealMatrix(int X, int Y);
		vector< vector< vector<dcomp> > > Init3DMatrix(int X, int Y, int Z);
		
		// functions multiplying scalars, vectors and matrices
		dcomp inner_product(vector<dcomp>&, vector<dcomp>&);
		double LOG2(double);
		void matrix_add_matrix(vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		void matrix_times_matrix(vector< vector<dcomp> >&, vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		void matrix_times_vector(vector<dcomp>&, vector< vector<dcomp> >&, vector<dcomp>&);
		void outer_product_add(vector< vector<dcomp> >&, vector<dcomp>&, vector<dcomp>&);
		void scalar_times_vector(vector<dcomp>&, double, vector<dcomp>&);
		void swap2D(vector< vector<dcomp> >&, vector< vector<dcomp> >&);
		void transpose_conjugate(vector< vector<dcomp> >&);
		
	private:
		int N, J, K;
		dcomp zero;
		vector<dcomp> rho2; /* 3rd dimension of rho */
		vector< vector<dcomp> > rho1; /* 2nd dimension of rho */
		vector< vector< vector<dcomp> > > rho; /* 1st dimension of rho. Dimension of rho is J x N x N */
		vector<dcomp> povm2; /* 3rd dimension of povm */
		vector< vector<dcomp> > povm1; /* 2nd dimension of povm */
		vector< vector< vector<dcomp> > > povm; /* 1st dimension of povm. Dimension of povm is K x N x N */		
};

MutInf::MutInf(int n, int j, int k)
	   :tol_MI(1.0e-15),tol_alpha(1E-13),tol_S(1E-15),N(n),J(j),K(k),zero(0.0,0.0),povmDimension(n),output("C:\\output.txt"),directGradChance(0)
{
 /* reserve enough memory space for rho and povm such as to avoid reallocation bug */
 /* The reason of the reallocation bug if memory is allocated dynamically using push_back() is as followed:
    The memory for the matrices are allocated continuously. If one of the dimension is increased, no such continuous memory space could be found, hence memory problem occurs.
    Alternatively, we can create a new matrix with the increased dimension and copy the entries of the old matrix to the new matrix, 
    but a lot of copying would be involved every time one of the dimensions is changed, hence we do not suggest it as a feasible method. */
	maxDimen = 30; // maximum dimension of the rho_j or povm_k matrix
	maxMatrix = 30; // maximum number of rho_j or povm_k to be used for computation
	
	// initialise rho and povm
	for(int s=0;s<maxDimen;s++) rho2.push_back(zero);
	for(int s=0;s<maxDimen;s++) rho1.push_back(rho2);
	for(int s=0;s<maxMatrix;s++) rho.push_back(rho1);
	
	for(int s=0;s<maxDimen;s++) povm2.push_back(zero);
	for(int s=0;s<maxDimen;s++) povm1.push_back(povm2);
	for(int s=0;s<maxMatrix;s++) povm.push_back(povm1);
	
	phi=0.5+(0.5*sqrt(5.0)); // the Golden ratio
}

// actual function that carry out the calculation of mutual information
void MutInf::calcMutualInformation()
{
	// initialisation of variables
	int i, j, k, l, cmpPOVM=0;
	dcomp ratio1, ratio2;
	dcomp temp(0,0);
	int restarted=0;
	int counter=0;
	double beta=0.0;
	int beta_count=0;
	int seed=(unsigned int)time(NULL);		
	new_MI=0.0; MI=0.0; alpha=1.0; 
	srand(seed);
	int test_import=0;
    
    // to determine the machine_epsilon
    double float_radix = 2.0;
    double inverse_radix = 1.0/float_radix;
    double machine_epsilon = 1.0;
    double temp_value = 1.0 + machine_epsilon;

    while(temp_value != 1.0) {
        machine_epsilon *= inverse_radix;
        temp_value = 1.0 + machine_epsilon;
    } // used later for the teimination criterion

	// initialise the output file
	ofstream outfile;
	outfile.open(output.c_str(), ios::out | ios::trunc);
	outfile << "Number of possible Quantum States J = " << J << "\n";
	outfile << "Number of POVM outcomes K = " << K << "\n";
	outfile << "Dimension of statistical operators N = " << N << "\n";
	outfile << "Percentage chance to calculate with direct gradient = "<< directGradChance << "/100 \n";
	outfile << "Tolerance in Mutual Information = " << tol_MI << "\n";
	outfile << "Starting seed for number generator = " << seed << "\n";
		
	// clear the probabilities stored previously
	prob_table.clear();
		
	vector< vector<dcomp> > A_dagger = Init2DMatrix(K,N);
	vector< vector<dcomp> > new_A_dagger = Init2DMatrix(K,N);
	vector< vector<dcomp> > S = Init2DMatrix(N,N);
	vector< vector<dcomp> > grad_MI = Init2DMatrix(K,N);
	vector< vector<dcomp> >	delta_A_dagger = Init2DMatrix(K,N);
	vector< vector<dcomp> > old_delta_A_dagger = Init2DMatrix(K,N);
	vector< vector<dcomp> > old_grad_MI = Init2DMatrix(K,N);
	old_grad_MI[0][0] = 1.0;
	vector< vector< vector<dcomp> > > R = Init3DMatrix(K,N,N);
	
    // check whether to randomise intial POVMs or not
	for(k=0; k<K; k++){
		for(i=0; i<N; i++){
			for(j=0; j<N; j++){
				if(povm[k][i][j]!=temp){test_import = 1; break;} // initial POVMs was manually input by the user, break out
			}
		}
	}

	if(test_import==1){ // import the manually input initial POVMs to S
               for(k=0; k<K; k++){
                      matrix_add_matrix(S, povm[k]);} // S is not zero matrix anymore
    } // otherwise do nothing, to randomly generate the initial POVMs
    
	gen_A_dagger(A_dagger, S); // initialise A_dagger 

restart:
		if (restarted==1)
		{
			counter = 0;
			seed = seed+1;
			srand(seed);
			gen_A_dagger(A_dagger, S);
		}
		restarted = 1;
		calc_prob_table(A_dagger); // for the specified rho, calculate the probability table using POVM A_dagger
		MI = calc_MI_from_prob_table(prob_table); // calculate the mutual information from the probability table
		if ((isnan(MI) || MI<0.0)) goto restart;

		while(1)
		{
			counter++;
			if (counter>10000) goto restart;
			calc_R(R);
			calc_grad_MI(grad_MI, R, A_dagger);
			
			// evaluate the chance to use direct gradient in the search for steepest ascent
			if((rand()%2)*100<directGradChance){
				for (i=0; i<K; i++){
					for (j=0; j<N; j++) {
						delta_A_dagger[i][j] = grad_MI[i][j];
						old_delta_A_dagger[i][j] = grad_MI[i][j];
					}
				}				
			}else{
				calc_delta_A_dagger_CG(beta, beta_count, grad_MI, old_grad_MI, delta_A_dagger, old_delta_A_dagger);
			}
			// to find the improved rho and povm
			find_alpha_GSS(new_A_dagger, A_dagger, delta_A_dagger);
			new_MI = calc_MI(alpha, new_A_dagger, A_dagger, delta_A_dagger);

			if ((isnan(new_MI) || new_MI<0.0)) goto restart;
			outfile << "counter = " << counter << ", new_MI = " << new_MI << "\n";
			if (2.0*(new_MI - MI) <= tol_MI*(new_MI+MI)+machine_epsilon && counter > 10) break; // termination criteria
			swap2D(new_A_dagger, A_dagger);
			MI = new_MI;
		}

		outfile << "\n\n";
		
		//---------------------------------------------
		//	print the input states to file		
		//---------------------------------------------		
		for(k=0; k<J; k++){
			outfile << "" << (k+1) << " = {";
			for(i=0; i<N; i++){
				outfile << "{";
				for(j=0; j<N; j++){
					outfile << real(rho[k][i][j]);
					if(imag(rho[k][i][j])>0)outfile << "+";
					if(imag(rho[k][i][j])!=0)outfile << imag(rho[k][i][j]) << "I";
					if(j!= N-1) outfile << ",";
				}
				outfile << "}";
				if(i!= N-1) outfile << ",";
			}
			outfile << "}\n";
		}
		outfile << "\n";
///////////////////////////////////////////////////////////////////

		// to form the POVM before elimination
		for(k=0; k<K; k++){
			outer_product_add(povm[k], A_dagger[k], A_dagger[k]);
		}

		//---------------------------------------------
		//	print the povm before elimination to file		
		//---------------------------------------------
		for(k=0; k<K; k++){
			outfile << "" << (k+1) << " = {";
			for(i=0; i<N; i++){
				outfile << "{";
				for(j=0; j<N; j++){
					outfile << real(povm[k][i][j]);
					if(imag(povm[k][i][j])>0)outfile << "+";
					if(imag(povm[k][i][j])!=0)outfile << imag(povm[k][i][j]) << "I";
					if(j!= N-1) outfile << ",";
				}
				outfile << "}";
				if(i!= N-1) outfile << ",";
			}
			outfile << "}\n";			
		}
		outfile << "\n";
////////////////////////////////////////////////////////////////////	
			
		outfile << "After elimininating the extra POVM outcomes\n";							
		//---------------------------------------------
		//  eliminate the extra povms
		//---------------------------------------------
		calc_prob_table(A_dagger); 
		double test, temp1, temp2, temp3;
		for(k=K-1; k>-1; k--){
			for(i=0; i<k; i++){
			    temp1=0;
                temp2=0; 
                temp3=0;
				for(j=0; j<J; j++){
					temp1 += prob_table[j][k]*prob_table[j][i];
					temp2 += prob_table[j][k]*prob_table[j][k];
					temp3 += prob_table[j][i]*prob_table[j][i];
				}

				test = 1.0 - temp1*temp1/(temp2*temp3);
				// if found two POVMs to be directly proportional to each other, add them up and assign to the ith POVM, move up the last POVM to the kth POVM being compared, reduced the total number of POVM, i.e. K
				if(test<0.001){
					matrix_add_matrix(povm[i], povm[k]);
					swap2D(povm[k], povm[K-1]);
					K--;
					break;
				}				
			}
		}

		//---------------------------------------------
		//	print the final povm to file		
		//---------------------------------------------
		for(k=0; k<K; k++){
			outfile << "" << (k+1) << " = {";
			for(i=0; i<N; i++){
				outfile << "{";
				for(j=0; j<N; j++){
					outfile << real(povm[k][i][j]);
					if(imag(povm[k][i][j])>0)outfile << "+";
					if(imag(povm[k][i][j])!=0)outfile << imag(povm[k][i][j]) << "I";
					if(j!=N-1) outfile << ",";
				}
				outfile << "}";
				if(i!=N-1) outfile << ",";
			}
			outfile << "}\n";			
		}
		// close the output file
		outfile.close();
}

//------------------------------------------------------------------------------
//	functions dealing with actual calculation
//------------------------------------------------------------------------------
// The calculated value of beta equals to gamma_i defined in Eq. (10.6.7) in p. 422 of NRC, or beta_k in Eq.(2.7.32) in p. 84 of NRC
double MutInf::calc_beta(vector< vector<dcomp> >& grad_MI, vector< vector<dcomp> >& old_grad_MI)
{
	double beta;
	int k;
	dcomp tempnum = 0.0;
	dcomp tempden = 0.0;
	for(k=0; k<K; k++){
		tempnum += inner_product(grad_MI[k], grad_MI[k])-inner_product(grad_MI[k], old_grad_MI[k]);
		tempden += inner_product(old_grad_MI[k], old_grad_MI[k]);
	}
	beta = real(tempnum)/real(tempden);
	return beta;
}

// This subroutine computes the conjugate gradient in a single iteration following the method in p. 83-86, or p. 420-423 of NRC
void MutInf::calc_delta_A_dagger_CG(double beta, int beta_count, vector< vector<dcomp> >& grad_MI, vector< vector<dcomp> >& old_grad_MI, vector< vector<dcomp> >& delta_A_dagger, vector< vector<dcomp> >& old_delta_A_dagger)
{
	/* Conjugate Gradient (CG) algorithm */
	int k, n;
	beta = calc_beta(grad_MI, old_grad_MI);
	dcomp tempsum = 0.0;
	if((beta_count < K*N*2) && (beta > 0)){
		for(k=0; k<K; k++){
			for(n=0; n<N; n++){
			    // this line correspond to Eq. (10.6.2) in NRC
				// delta_A_dagger corresponds to h_{i+1} in Eq. (10.6.2), or p_{k+1} in Eq. (2.7.32) in NRC
				// grad_MI corresponds to g_{i+1} in Eq. (10.6.2), or r_{k+1} in Eq. (2.7.32) in NRC
				delta_A_dagger[k][n] = grad_MI[k][n] + beta * old_delta_A_dagger[k][n];
			}
		}
		
		for(k=0; k<K; k++){
			tempsum += inner_product(delta_A_dagger[k], grad_MI[k]);
		}
		// check that inner_product(delta_A_dagger, grad_MI) is positive
		if(real(tempsum) > 0 ){
			beta_count++;
		}else{
			swap2D(delta_A_dagger, grad_MI);
			beta_count=0;
		}
	}
	else{
		swap2D(delta_A_dagger, grad_MI);
		beta_count=0;
	}

	// backup grad_MI to old_grad_MI; after that it doesn't matter what happens to grad_MI so we can just swap
	swap2D(grad_MI, old_grad_MI);
	
	// backup delta_A_dagger to old_delta_A_dagger; cannot swap because delta_A_dagger is required for calculating new_A_dagger 
	for(k=0;k<K;k++){
		for(n=0;n<N;n++){
			old_delta_A_dagger[k][n] = delta_A_dagger[k][n];
		}
	}
}

void MutInf::calc_grad_MI(vector< vector<dcomp> >& m, vector< vector< vector<dcomp> > >& a, vector< vector<dcomp> >& b)
{
	int i, j, k;
	vector< vector<dcomp> > GAMMA = Init2DMatrix(N, N);
	vector< vector<dcomp> > R_minus_GAMMA = Init2DMatrix(N, N);
	vector<dcomp> RA_dagger(N, 0);
	
	for(k=0; k<K; k++){
		matrix_times_vector(RA_dagger, a[k], b[k]);
		outer_product_add(GAMMA, RA_dagger, b[k]);
	}
	transpose_conjugate(GAMMA);

	for(k=0; k<K; k++){
		// subtract GAMMA from R
		for(i=0; i<N; i++){
			for(j=0; j<N; j++){
				R_minus_GAMMA[i][j] = a[k][i][j] - GAMMA[i][j];
			}
		}
		matrix_times_vector(m[k], R_minus_GAMMA, b[k]);
	}
	// when this routine is executed till this line, grad_MI[k] = (R_k - \sum_l \pi_l R_l)A_dagger_k
}

void MutInf::calc_L_inverse(vector< vector<dcomp> >& L, vector< vector<dcomp> >& L_inv, vector< vector<dcomp> >& S)
{
	// calculate L where S = L x L_dagger, where L is a lower triangular matrix, then find L_inverse by back substitution
	// first calculate L
	// L_ij = 1/L_jj (S_ij - \sum_{k=0}^{j-1}L_ik L*_jk), i>j
	// L_ii = \sqrt{S_ii - \sum_{k=0}^{i-1} L_{ik}L*_{jk}}
	int row, col, k;
	dcomp sum;
	// This routine computes the LU decomposition of S. Since S is hermitian, U=dagger of L.
    // The computation is well explained in section 2.3 of NRC, p. 43-49.
	for(row=0; row<N; row++)
	{
		for(col=0; col<row+1; col++)
		{
			// first obtain summation \sum_{k=0}^{j-1}L_ik L*_jk
			sum = 0.0;
			for(k=0; k<col; k++)
			{
				sum = sum + L[row][k] * conj(L[col][k]);
			}

			if(col < row)
			{
				L[row][col] = (S[row][col] - sum) / conj(L[col][col]);
			}
			else
			{
				L[row][col] = sqrt(S[row][col] - sum);
			}
		}
	}
	
	// now calculate L_inverse, assuming that L_inverse is a lower triangular matrix as well
	// for i!=j, \sum_{k=j}^{k=i-1} L_ik L^{-1}_kj = - L_ii L^{-1}_ij
	for(col=0; col<N; col++)
	{
		L_inv[col][col] = 1.0 / L[col][col];
		for(row=col+1; row<N; row++)
		{
			sum = 0;
			for(k=col; k<row; k++)
			{
				sum = sum + L[row][k] * L_inv[k][col]; 
			}
			L_inv[row][col] = -sum / L[row][row];
		}
	}
}

inline double MutInf::calc_MI(double some_alpha, vector< vector<dcomp> >& new_A_dagger, vector< vector<dcomp> >& A_dagger, vector< vector<dcomp> >& delta_A_dagger)
{
	calc_new_A_dagger(some_alpha, new_A_dagger, A_dagger, delta_A_dagger);
	calc_prob_table(new_A_dagger);
	return calc_MI_from_prob_table(prob_table);
}

double MutInf::calc_MI_from_prob_table(vector< vector<double> >& a)
{
	int j, k;
	double temp = 0.0;
	for(j=0; j<J; j++)
	{
		for(k=0; k<K; k++)
		{
			temp = temp + a[j][k] * ( LOG2(a[j][k]) - LOG2(a[j][K]) - LOG2(a[J][k]) );
		}
	}
	return temp;
}

void MutInf::calc_new_A_dagger(double some_alpha, vector< vector<dcomp> >& new_A_dagger, vector< vector<dcomp> >& A_dagger, vector< vector<dcomp> >& delta_A_dagger)
{
	int k, n;
	vector< vector<dcomp> > S = Init2DMatrix(N,N);
	vector< vector<dcomp> > tmp_S_inverse_sqrt = Init2DMatrix(N,N);
	vector< vector<dcomp> > S_inverse_sqrt = Init2DMatrix(N,N);	
	vector< vector<dcomp> > tmp_A_dagger = Init2DMatrix(K,N);	
	
	for(k=0; k<K; k++){
		for(n=0; n<N; n++){
			new_A_dagger[k][n] = A_dagger[k][n] + some_alpha * delta_A_dagger[k][n];
		}
	}

	for(k=0; k<K; k++){
		outer_product_add(S, new_A_dagger[k], new_A_dagger[k]);
	}
	
	// calc_S_inverse_sqrt();
	calc_L_inverse(tmp_S_inverse_sqrt, S_inverse_sqrt, S);

	for(k=0; k<K; k++){
		matrix_times_vector(tmp_A_dagger[k], S_inverse_sqrt, new_A_dagger[k]);
		scalar_times_vector(new_A_dagger[k], 1.0, tmp_A_dagger[k]);
	}
}

void MutInf::calc_prob_table(vector< vector<dcomp> >& any_A_dagger)
{
	int j, k;
	double temp;
	vector< vector<dcomp> > tmp_A_dagger = Init2DMatrix(K,N);
	prob_table = Init2DRealMatrix(J+1, K+1);
	
	for(j=0; j<J; j++)
	{
		temp = 0.0;
		for(k=0; k<K; k++)
		{
			matrix_times_vector(tmp_A_dagger[k], rho[j], any_A_dagger[k]);
			prob_table[j][k]=real(inner_product(any_A_dagger[k], tmp_A_dagger[k]));
			if (prob_table[j][k] < 0.0) prob_table[j][k] = 0.0;
			temp += prob_table[j][k];
		}
		prob_table[j][K] = temp;
	}
	for(k=0; k<K; k++)
	{
		temp = 0.0;
		for(j=0; j<J; j++)
		{
			temp += prob_table[j][k];
		}
		prob_table[J][k] = temp;
	}
}

void MutInf::calc_R(vector< vector< vector<dcomp> > >& a) // R depends on rho and prob_table
{
	int j, k, row, col;
	for(k=0; k<K; k++){
		for(row=0; row<N; row++){
			for(col=0; col<N; col++){
				a[k][row][col]=0.0;
				for(j=0; j<J; j++){
					a[k][row][col] += rho[j][row][col] * LOG2( prob_table[j][k] / (prob_table[j][K] * prob_table[J][k]) );
				}
			}
		}
	}
}

// Given the direction of change, i.e. delta_A_dagger, this routine attempts to find the size of step to take in the direction of change, i.e. alpha.
// The computation of alpha is carried out following the routine golden() in p. 401-402 of NRC. The idea of the algorithm is explained in section 10.2 from p. 397-400 NRC.
void MutInf::find_alpha_GSS(vector< vector<dcomp> >& new_A_dagger, vector< vector<dcomp> >& A_dagger, vector< vector<dcomp> >& delta_A_dagger)
{
	double alphaR, alphaL, MIR, MIL, interval, current_max;
	alpha = 1.0;
	new_MI = calc_MI(alpha, new_A_dagger, A_dagger, delta_A_dagger);
	
	/* find an upper bound for the optimal alpha */
	if(new_MI > MI)
	{
		current_max = new_MI;
		while(1)
		{
			/* increase alpha until MI(alpha) < MI */
			alpha = alpha + 1.0;
			new_MI = calc_MI(alpha, new_A_dagger, A_dagger, delta_A_dagger);
			if(isnan(new_MI) || new_MI < current_max)
			{
				alpha = alpha - 1.0; break; // p. 399 in NRC
			}
			else current_max = new_MI;
			if(alpha > 100.0) break; // limit alpha
		}
	}
	else
	{
		while(1)
		{
			/* half alpha until MI(alpha) > MI */
			alpha = alpha * 0.5;
			if (alpha < tol_alpha) break;
			new_MI = calc_MI(alpha, new_A_dagger, A_dagger, delta_A_dagger);
			if(new_MI > MI) {alpha = alpha * 2.0; break;}
		}
	}

	/* apply Golden Section Search */
	alphaR = phi * alpha;
	alphaL = (phi-1.0) * alpha;
	MIR = calc_MI(alphaR, new_A_dagger, A_dagger, delta_A_dagger);
	MIL = calc_MI(alphaL, new_A_dagger, A_dagger, delta_A_dagger);
	interval = alpha / (2.0*phi + 1.0);
	while(interval > tol_alpha * (alphaR + alphaL))
	{
		interval = interval * (phi-1.0);
		if (MIR > MIL)
		{
			alphaL = alphaR;
			MIL = MIR;
			alphaR = alphaR + interval;
			MIR = calc_MI(alphaR, new_A_dagger, A_dagger, delta_A_dagger);
		}
		else
		{
			alphaR = alphaL;
			MIR = MIL;
			alphaL = alphaL - interval;
			MIL = calc_MI(alphaL, new_A_dagger, A_dagger, delta_A_dagger);
		}
	}
	alpha = (alphaR + alphaL) * 0.5;
}

void MutInf::gen_A_dagger(vector< vector<dcomp> >& A_dagger, vector< vector<dcomp> >& S)
{
	int i, j, k, n;
	vector< vector<dcomp> > tmp_A_dagger = Init2DMatrix(K, N);
	vector< vector<dcomp> > tmp_S_inverse_sqrt = Init2DMatrix(N, N);
	vector< vector<dcomp> > S_inverse_sqrt = Init2DMatrix(N, N);
	
	for(n=0; n<N; n++){
		A_dagger[n][n] = 1.0;
	}
	
	for(k=0; k<K; k++){
		for(n=0; n<N; n++){
			dcomp random = complex<double>(0.002*((rand()/(double)RAND_MAX) - 0.5), 0.002*((rand()/(double)RAND_MAX)-0.5));
			A_dagger[k][n] = A_dagger[k][n] + random;
		}
	}
	
	for(i=0; i<N; i++){
		for(j=0; j<N; j++){
			S[i][j] = 0.0;
		}
	}

	int test = 0;
	for(k=0; k<K; k++){
		outer_product_add(S, A_dagger[k], A_dagger[k]);
	}
	
	calc_L_inverse(tmp_S_inverse_sqrt, S_inverse_sqrt, S);

	for(k=0; k<K; k++){
		matrix_times_vector(tmp_A_dagger[k], S_inverse_sqrt, A_dagger[k]);
		scalar_times_vector(A_dagger[k], 1.0, tmp_A_dagger[k]);
	}	
}

//------------------------------------------------------------------------------
//	functions for interaction with GUI
//------------------------------------------------------------------------------
int MutInf::getK()
{
	return K;
}

vector< vector<dcomp> > MutInf::getPOVM(int X)
{
	return povm[X];
}

vector< vector<dcomp> > MutInf::getRho(int X)
{
	return rho[X];
}

void MutInf::setRhoElement(int x, int y, int z, double d, int ri)
{
	if(ri == 0){
		dcomp temp(d,imag(rho[x][y][z]));
		rho[x][y][z] = temp;
	}else{
		dcomp temp(real(rho[x][y][z]),d);
		rho[x][y][z] = temp;
	}
}

void MutInf::setPOVMElement(int x, int y, int z, double d, int ri)
{
	if(ri == 0){
		dcomp temp(d,imag(povm[x][y][z]));
		povm[x][y][z] = temp;
	}else{
		dcomp temp(real(povm[x][y][z]),d);
		povm[x][y][z] = temp;
	}
}

void MutInf::UpdatePOVMDimension(int k, int n)
{
	int x, y, z;
	for(x=0; x<maxMatrix; x++){
		for(y=0; y<maxDimen; y++){
			for(z=0; z<maxDimen; z++){
				povm[x][y][z] = zero;
			}
		}
	}
	
	if(k<n){
		K = n;
	}else{
		K = k;
	} 
	povmDimension = n;
}

void MutInf::UpdateRhoDimension(int j, int n)
{
	int x, y, z;
	for(x=0; x<maxMatrix; x++){
		for(y=0; y<maxDimen; y++){
			for(z=0;z<maxDimen;z++){
				if(!(x<j && y<n && z<n))rho[x][y][z] = zero;
			}
		}
	}
	N = n;
	J = j;
}

//-------------------------------------------------------------------------------------
//	functions to initialize 2D and 3D zero matrices, with all entries set to 0
//-------------------------------------------------------------------------------------
vector< vector<dcomp> > MutInf::Init2DMatrix(int X, int Y)
{
	vector<dcomp> b(Y, 0); /* 2nd dimension */
	vector< vector<dcomp> > a(X, b);
	
	return a;
}

vector< vector<double> > MutInf::Init2DRealMatrix(int X, int Y)
{
	vector<double> b(Y, 0); /* 2nd dimension */
	vector< vector<double> > a(X, b);
	
	return a;
}

vector< vector< vector<dcomp> > > MutInf::Init3DMatrix(int X, int Y, int Z)
{
	vector<dcomp> c(Z, 0); /* 3rd dimension */
	vector< vector<dcomp> > b(Y, c); /* 2nd dimension */
	vector< vector< vector<dcomp> > > a(X, b);
	
	return a;
}
//------------------------------------------------------------------------------
//	functions to multiply scalar, vectors and matrices
//------------------------------------------------------------------------------

dcomp MutInf::inner_product(vector<dcomp>& v1, vector<dcomp>& v2)
{
	int i;
	dcomp temp = 0.0;
	for(i=0; i<N; i++)
	{
		temp += conj(v1[i]) * v2[i];
	}
	return temp;
}

double MutInf::LOG2(double x)
{
	if(x==0.0) return 0.0;
	return log(x)/log(2.0);
}

// add two N by N matrix m2 and m3 and assign to m2;
void MutInf::matrix_add_matrix(vector< vector<dcomp> >& m2, vector< vector<dcomp> >& m3)
{
	int i, j;
	for(i=0; i<N; i++)	
	{
		for(j=0; j<N; j++)	
		m2[i][j] += m3[i][j];		
	}
}

// multiply two N by N matrix m2 and m3 and assign the product to m1
void MutInf::matrix_times_matrix(vector< vector<dcomp> >& m1, vector< vector<dcomp> >& m2, vector< vector<dcomp> >& m3)
{
	int i, j, k;
	for(i=0; i<N; i++)
	{
		for(k=0; k<N; k++)
		{
			m1[i][k] = 0;
			for(j=0; j<N; j++)
			{
				m1[i][k] += m2[i][j] * m3[j][k];
			}
		}
	}
}

// multiply an N by N matrix m with an N-element vector v2 and assign the product to v1
void MutInf::matrix_times_vector(vector<dcomp>& v1, vector< vector<dcomp> >& m, vector<dcomp>& v2)
{
	int i, j;
	for(i=0; i<N; i++)
	{
		v1[i] = 0;
		for(j=0; j<N; j++)
		{
			v1[i] += m[i][j] * v2[j];
		}
	}
}

void MutInf::outer_product_add(vector< vector<dcomp> >& m, vector<dcomp>& v1, vector<dcomp>& v2)
{
	int row, col;
	dcomp temp;
	for(col=0; col<N; col++)
	{
		temp = conj(v2[col]);
		for(row=0; row<N; row++)
			m[row][col] += v1[row] * temp;
	}
}

// multiply an N-element vector v2 with a scalar s and assign the product to an N-element vector v1
void MutInf::scalar_times_vector(vector<dcomp>& v1, double s, vector<dcomp>& v2)
{
	int i;
	for(i=0; i<N; i++)
	{
		v1[i] = s * v2[i];
	}
}

void MutInf::swap2D(vector< vector<dcomp> >& m1, vector< vector<dcomp> >& m2)
{
	m1.swap(m2);
}

void MutInf::transpose_conjugate(vector< vector<dcomp> >& m)
{
	int row, col;
	dcomp temp;
	for(row=0; row<N; row++){
		for(col=0; col <= row; col++){
			temp = m[row][col];
			m[row][col] = conj(m[col][row]);
			m[col][row] = conj(temp);
		}
	}
}
